@@ -19,31 +19,49 @@ setup_logging()
 log = logging.getLogger("tdub-cli")
 
 
-@click.group(context_settings=dict(max_content_width=82))
+@click.group(context_settings=dict(max_content_width=82, help_option_names=['-h', '--help']))
 def cli():
     pass
 
 
-@cli.command("train-single", context_settings=dict(max_content_width=92))
-@click.argument("datadir", type=click.Path(exists=True))
-@click.argument("region", type=str)
-@click.argument("outdir", type=click.Path())
+@cli.group("train")
+def train():
+    """Tasks to perform machine learning steps."""
+    pass
+
+
+@cli.group("apply")
+def cli_apply():
+    """Tasks to apply machine learning models to data."""
+    pass
+
+
+@cli.group("rex")
+def rex():
+    """Tasks interacting with TRExFitter results."""
+    pass
+
+
+@cli.group("misc")
+def misc():
+    """Tasks under a miscellaneous umbrella."""
+
+
+@train.command("prep")
+@click.argument("datadir", type=click.Path(resolve_path=True, exists=True))
+@click.argument("region", type=click.Choice(["1j1b", "2j1b", "2j2b"]))
+@click.argument("outdir", type=click.Path(resolve_path=True))
 @click.option("-p", "--pre-exec", type=click.Path(resolve_path=True), help="Python code to pre-execute")
 @click.option("-n", "--nlo-method", type=str, default="DR", help="tW simluation NLO method", show_default=True)
 @click.option("-x", "--override-selection", type=str, help="override selection with contents of file")
 @click.option("-t", "--use-tptrw", is_flag=True, help="apply top pt reweighting")
 @click.option("-r", "--use-trrw", is_flag=True, help="apply top recursive reweighting")
 @click.option("-i", "--ignore-list", type=str, help="variable ignore list file")
-@click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
-@click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
+@click.option("-m", "--multiple-ttbar-samples", is_flag=True, help="use multiple ttbar MC samples")
+@click.option("-a", "--use-inc-af2", is_flag=True, help="use inclusive af2 samples")
+@click.option("-f", "--bkg-sample-frac", type=float, help="use a fraction of the background")
 @click.option("-d", "--use-dilep", is_flag=True, help="train with dilepton samples")
-@click.option("-k", "--use-sklearn", is_flag=True, help="use sklearn instead of lgbm")
-@click.option("--learning-rate", type=float, default=0.1, help="learning_rate model parameter", show_default=True)
-@click.option("--num-leaves", type=int, default=31, help="num_leaves model parameter", show_default=True)
-@click.option("--min-child-samples", type=int, default=50, help="min_child_samples model parameter", show_default=True)
-@click.option("--max-depth", type=int, default=5, help="max_depth model parameter", show_default=True)
-@click.option("--reg-lambda", type=float, default=0, help="lambda (L2) regularization", show_default=True)
-def single(
+def train_prep(
     datadir,
     region,
     outdir,
@@ -53,27 +71,30 @@ def single(
     use_tptrw,
     use_trrw,
     ignore_list,
-    test_size,
-    early_stop,
+    multiple_ttbar_samples,
+    use_inc_af2,
+    bkg_sample_frac,
     use_dilep,
-    use_sklearn,
-    learning_rate,
-    num_leaves,
-    min_child_samples,
-    max_depth,
-    reg_lambda,
 ):
-    """Execute a single training round."""
+    """Prepare data for training."""
+
     if pre_exec is not None:
         exec(PosixPath(pre_exec).read_text())
 
-    from tdub.ml_train import single_training, prepare_from_root
+    from tdub.ml_train import prepare_from_root, persist_prepared_data
     from tdub.data import avoids_for, quick_files
     from tdub.frames import drop_cols
 
     qf = quick_files(datadir)
     sig_files = qf[f"tW_{nlo_method}"] if use_dilep else qf[f"tW_{nlo_method}_inc"]
-    bkg_files = qf["ttbar"] if use_dilep else qf["ttbar_inc"]
+    if multiple_ttbar_samples:
+        bkg_files = qf["ttbar_inc_AFII"] + qf["ttbar_PS"]
+    elif use_inc_af2:
+        sig_files = qf[f"tW_{nlo_method}_inc_AFII"]
+        bkg_files = qf["ttbar_inc_AFII"]
+    else:
+        bkg_files = qf["ttbar"] if use_dilep else qf["ttbar_inc"]
+
     override_sel = override_selection
     if override_sel:
         override_sel = PosixPath(override_sel).read_text().strip()
@@ -85,11 +106,63 @@ def single(
         override_selection=override_sel,
         use_tptrw=use_tptrw,
         use_trrw=use_trrw,
+        bkg_sample_frac=bkg_sample_frac,
     )
     drop_cols(df, *avoids_for(region))
     if ignore_list:
         drops = PosixPath(ignore_list).read_text().strip().split()
         drop_cols(df, *drops)
+    outdir = PosixPath(outdir)
+    persist_prepared_data(outdir, df, y, w)
+    (outdir / "region.txt").write_text(f"{region}\n")
+    (outdir / "nlo_method.txt").write_text(f"{nlo_method}\n")
+    (outdir / "files_sig.txt").write_text("{}\n".format(("\n".join(sig_files))))
+    (outdir / "files_bkg.txt").write_text("{}\n".format(("\n".join(bkg_files))))
+
+
+@train.command("single")
+@click.argument("datadir", type=click.Path(resolve_path=True, exists=True))
+@click.argument("outdir", type=click.Path(resolve_path=True))
+@click.option("-p", "--pre-exec", type=click.Path(exists=True, resolve_path=True), help="Python code to pre-execute")
+@click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
+@click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
+@click.option("-k", "--use-sklearn", is_flag=True, help="use sklearn instead of lgbm")
+@click.option("-g", "--use-xgboost", is_flag=True, help="use xgboost instead of lgbm")
+@click.option("-l", "--learning-rate", type=float, default=0.1, help="learning_rate model parameter", show_default=True)
+@click.option("-n", "--num-leaves", type=int, default=16, help="num_leaves model parameter", show_default=True)
+@click.option("-m", "--min-child-samples", type=int, default=500, help="min_child_samples model parameter", show_default=True)
+@click.option("-d", "--max-depth", type=int, default=5, help="max_depth model parameter", show_default=True)
+@click.option("-r", "--reg-lambda", type=float, default=0, help="lambda (L2) regularization", show_default=True)
+def train_single(
+    datadir,
+    outdir,
+    pre_exec,
+    test_size,
+    early_stop,
+    use_sklearn,
+    use_xgboost,
+    learning_rate,
+    num_leaves,
+    min_child_samples,
+    max_depth,
+    reg_lambda,
+):
+    """Execute single training round."""
+
+    if pre_exec is not None:
+        exec(PosixPath(pre_exec).read_text())
+
+    from tdub.ml_train import single_training
+    import pandas as pd
+    import numpy as np
+
+    datadir = PosixPath(datadir)
+    df = pd.read_hdf(datadir / "df.h5", "df")
+    y = np.load(datadir / "labels.npy")
+    w = np.load(datadir / "weights.npy")
+    df.selection_used = (
+        datadir / "selection.txt"
+    ).read_text().strip()
     train_axes = dict(
         learning_rate=learning_rate,
         num_leaves=num_leaves,
@@ -97,7 +170,10 @@ def single(
         max_depth=max_depth,
         reg_lambda=reg_lambda,
     )
-    extra_sum = {"region": region, "nlo_method": nlo_method}
+    extra_sum = {
+        "region": PosixPath(datadir / "region.txt").read_text().strip(),
+        "nlo_method": PosixPath(datadir / "nlo_method.txt").read_text().strip(),
+    }
     single_training(
         df,
         y,
@@ -108,38 +184,24 @@ def single(
         early_stopping_rounds=early_stop,
         extra_summary_entries=extra_sum,
         use_sklearn=use_sklearn,
+        use_xgboost=use_xgboost,
     )
-    return 0
 
 
-@cli.command("train-scan", context_settings=dict(max_content_width=140))
+@train.command("scan")
 @click.argument("datadir", type=click.Path(exists=True, resolve_path=True))
-@click.argument("region", type=str)
 @click.argument("workspace", type=click.Path(exists=False))
 @click.option("-p", "--pre-exec", type=click.Path(resolve_path=True), help="Python code to pre-execute")
-@click.option("-n", "--nlo-method", type=str, default="DR", help="tW simluation NLO method", show_default=True)
 @click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
-@click.option("-x", "--override-selection", type=str, help="override selection with contents of file")
-@click.option("-t", "--use-tptrw", is_flag=True, help="apply top pt reweighting")
-@click.option("-r", "--use-trrw", is_flag=True, help="apply top recursive reweighting")
-@click.option("-i", "--ignore-list", type=str, help="variable ignore list file")
 @click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
-@click.option("-d", "--use-dilep", is_flag=True, help="train with dilepton samples")
 @click.option("--overwrite", is_flag=True, help="overwrite existing workspace")
 @click.option("--and-submit", is_flag=True, help="submit the condor jobs")
-def scan(
+def train_scan(
     datadir,
-    region,
     workspace,
     pre_exec,
-    nlo_method,
     early_stop,
-    override_selection,
-    use_tptrw,
-    use_trrw,
-    ignore_list,
     test_size,
-    use_dilep,
     overwrite,
     and_submit,
 ):
@@ -148,20 +210,13 @@ def scan(
     DATADIR points to the intput ROOT files, training is performed on
     the REGION and all output is saved to WORKSPACE.
 
-    $ tdub train-scan /data/path 2j2b scan_2j2b
+    $ tdub train scan /data/path 2j2b scan_2j2b
 
     """
 
     if pre_exec is not None:
         exec(PosixPath(pre_exec).read_text())
 
-    # from tdub.batch import (
-    #     create_condor_workspace,
-    #     condor_preamble,
-    #     add_condor_arguments,
-    #     condor_submit,
-    # )
-
     from tdub.batch import create_condor_workspace
     import tdub.config
     import itertools
@@ -169,23 +224,8 @@ def scan(
     ws = create_condor_workspace(workspace, overwrite=overwrite)
     (ws / "res").mkdir()
 
-    log.info(f"Preparing a scan; results saved to {workspace}")
-    log.info(f"  - region: {region}")
-    log.info(f"  - NLO method: {nlo_method}")
-    log.info("  - Using {} samples".format("dilepton" if use_dilep else "inclusive"))
-    log.info("  - Apply top pt reweight: {}".format("yes" if use_tptrw else "no"))
-
     runs = []
     i = 0
-    override_sel = override_selection
-    if override_sel is None:
-        override_sel = "_NONE"
-    else:
-        override_sel = str(PosixPath(override_sel).resolve())
-    if ignore_list is None:
-        ignore_list = "_NONE"
-    else:
-        ignore_list = str(PosixPath(ignore_list).resolve())
     if pre_exec is None:
         pre_exec = "_NONE"
     else:
@@ -213,37 +253,25 @@ def scan(
         )
         outdir = ws / "res" / f"{i:04d}_{suffix}"
         arglist = (
-            "{} {} {} -s {} -n {} -x {} -i {} -p {} "
+            "{} {} -p {} -s {} "
             "--learning-rate {} "
             "--num-leaves {} "
             "--min-child-samples {} "
             "--max-depth {} "
             "--reg-lambda {} "
             "--early-stop {} "
-            "{}"
-            "{}"
-            "{}"
         ).format(
             datadir,
-            region,
             outdir,
-            test_size,
-            nlo_method,
-            override_sel,
-            ignore_list,
             pre_exec,
+            test_size,
             learning_rate,
             num_leaves,
             min_child_samples,
             max_depth,
             reg_lambda,
             early_stop,
-            "-r " if use_trrw else "",
-            "-t " if use_tptrw else "",
-            "-d " if use_dilep else "",
         )
-        arglist = arglist.replace("-x _NONE ", "")
-        arglist = arglist.replace("-i _NONE ", "")
         arglist = arglist.replace("-p _NONE ", "")
         runs.append(arglist)
         i += 1
@@ -251,7 +279,7 @@ def scan(
     with (ws / "run.sh").open("w") as outscript:
         print("#!/bin/bash\n\n", file=outscript)
         for run in runs:
-            print(f"tdub train-single {run}\n", file=outscript)
+            print(f"tdub train single {run}\n", file=outscript)
     os.chmod(ws / "run.sh", 0o755)
 
     import pycondor
@@ -271,7 +299,7 @@ def scan(
         dag=condor_dag,
     )
     for run in runs:
-        condor_job_scan.add_arg(f"train-single {run}")
+        condor_job_scan.add_arg(f"train single {run}")
     condor_job_check = pycondor.Job(
         name="job_train_check",
         universe="vanilla",
@@ -285,7 +313,7 @@ def scan(
         log=str(ws / "log"),
         dag=condor_dag,
     )
-    condor_job_check.add_arg(f"train-check {ws}")
+    condor_job_check.add_arg(f"train check {ws}")
     condor_job_check.add_parent(condor_job_scan)
 
     if and_submit:
@@ -304,11 +332,11 @@ def scan(
     return 0
 
 
-@cli.command("train-check", context_settings=dict(max_content_width=92))
+@train.command("check")
 @click.argument("workspace", type=click.Path(exists=True))
 @click.option("-p", "--print-top", is_flag=True, help="Print the top results")
 @click.option("-n", "--n-res", type=int, default=10, help="Number of top results to print", show_default=True)
-def check(workspace, print_top, n_res):
+def train_check(workspace, print_top, n_res):
     """Check the results of a parameter scan WORKSPACE."""
     from tdub.ml_train import SingleTrainingSummary
     import shutil
@@ -356,13 +384,12 @@ def check(workspace, print_top, n_res):
     return 0
 
 
-@cli.command("train-fold", context_settings=dict(max_content_width=92))
+@train.command("fold")
 @click.argument("scandir", type=click.Path(exists=True))
 @click.argument("datadir", type=click.Path(exists=True))
 @click.option("-t", "--use-tptrw", is_flag=True, help="use top pt reweighting")
-@click.option("-r", "--random-seed", type=int, default=414, help="random seed for folding", show_default=True)
 @click.option("-n", "--n-splits", type=int, default=3, help="number of splits for folding", show_default=True)
-def fold(scandir, datadir, use_tptrw, random_seed, n_splits):
+def train_fold(scandir, datadir, use_tptrw, n_splits):
     """Perform a folded training based on a hyperparameter scan result."""
     from tdub.ml_train import folded_training, prepare_from_root
     from tdub.data import quick_files
@@ -401,12 +428,12 @@ def fold(scandir, datadir, use_tptrw, random_seed, n_splits):
         {"verbose": 10},
         str(outdir),
         summary["region"],
-        kfold_kw={"n_splits": n_splits, "shuffle": True, "random_state": random_seed},
+        kfold_kw={"n_splits": n_splits, "shuffle": True},
     )
     return 0
 
 
-@cli.command("apply-single", context_settings=dict(max_content_width=92))
+@cli_apply.command("single")
 @click.argument("infile", type=click.Path(exists=True))
 @click.argument("arrname", type=str)
 @click.argument("outdir", type=click.Path())
@@ -459,7 +486,7 @@ def apply_single(infile, arrname, outdir, fold_results=None, single_results=None
     np.save(npyfilename, result_arr)
 
 
-@cli.command("apply-all", context_settings=dict(max_content_width=92))
+@cli_apply.command("all")
 @click.argument("datadir", type=click.Path(exists=True))
 @click.argument("arrname", type=str)
 @click.argument("outdir", type=click.Path(resolve_path=True))
@@ -515,7 +542,7 @@ def apply_all(
         dag=condor_dag,
     )
     for run in arglist:
-        condor_job_scan.add_arg(f"apply-single {run}")
+        condor_job_scan.add_arg(f"apply single {run}")
 
     if and_submit:
         condor_dag.build_submit()
@@ -523,7 +550,67 @@ def apply_all(
         condor_dag.build()
 
 
-@cli.command("soverb", context_settings=dict(max_content_width=92))
+@train.command("itables")
+@click.argument("summary-file", type=click.Path(exists=True))
+def itables(summary_file):
+    """Generate importance tables."""
+    import tdub.config
+    import json
+    from textwrap import dedent
+    tdub.config.init_meta_table()
+    summary = json.loads(PosixPath(summary_file).read_text())
+    imp_gain = summary["importances_gain"]
+    imp_split = summary["importances_split"]
+    names = imp_gain.keys()
+    imp_gain = [round(x, 4) for x in imp_gain.values()]
+    imp_split = [round(x, 4) for x in imp_split.values()]
+    imp_gain, names, imp_split = (list(reversed(t)) for t in zip(*sorted(zip(imp_gain, names, imp_split))))
+    print(dedent("""\
+    \\begin{table}[htbp]
+      \\begin{center}
+        \\caption{XXX}
+        \\label{XXX}
+        \\begin{tabular}{lcc}
+          \\toprule
+          Variable & Importance (gain) & Importance (split) \\\\
+          \\midrule"""))
+    for n, g, s in zip(names, imp_gain, imp_split):
+        print("      {} & {} & {} \\\\".format(tdub.config.PLOTTING_META_TABLE["titles"][n]["mpl"], g, s))
+    print(dedent("""\
+          \\bottomrule
+        \\end{tabular}
+      \\end{center}
+    \\end{table}
+    """))
+
+
+@rex.command("stacks")
+@click.argument("workspace", type=click.Path(exists=True))
+@click.option("--chisq/--no-chisq", default=True, help="Do or don't print chi-square information.")
+@click.option("-n", "--n-test", type=int, default=-1, help="Test only n plots (for stacks).")
+def rex_stacks(workspace, chisq, n_test):
+    """Generate plots from TRExFitter result."""
+    import tdub.rex
+    import tdub.config
+    outdir = PosixPath(workspace) / "matplotlib"
+    outdir.mkdir(exist_ok=True)
+    tdub.config.init_meta_table()
+    tdub.config.init_meta_logy()
+    tdub.rex.plot_all_regions(workspace, outdir, stage="pre", show_chisq=chisq, n_test=n_test)
+    tdub.rex.plot_all_regions(workspace, outdir, stage="post", show_chisq=chisq, n_test=n_test)
+    return 0
+
+
+@rex.command("impact")
+@click.argument("workspace", type=click.Path(exists=True))
+def rex_impact(workspace, outdir):
+    """Generate impact plot from TRExFitter result."""
+    import tdub.rex
+    tdub.rex.nuispar_impact_plot_top15(workspace)
+    return 0
+
+
+@misc.command("soverb")
 @click.argument("datadir", type=click.Path(exists=True))
 @click.argument("selections", type=click.Path(exists=True))
 @click.option("-t", "--use-tptrw", is_flag=True, help="use top pt reweighting")
@@ -567,66 +654,6 @@ def soverb(datadir, selections, use_tptrw):
         print(sel, s_df["weight_nominal"].sum() / b_df["weight_nominal"].sum())
 
 
-@cli.command("rex-plot", context_settings=dict(max_content_width=92))
-@click.argument("workspace", type=click.Path(exists=True))
-@click.option("-o", "--outdir", type=str, default="auto", help="Manual output directory.")
-@click.option("-n", "--n-test", type=int, default=-1, help="Test only n plots (for stacks).")
-@click.option("--stacks/--no-stacks", default=True, help="Do or don't plot stacks.")
-@click.option("--chisq/--no-chisq", default=True, help="Do or don't print chi-square information.")
-@click.option("--impact/--no-impact", default=True, help="Do or don't produce the impact plot.")
-def rex_plot(workspace, outdir, stacks, chisq, impact, n_test):
-    """Generate plots from TRExFitter WORKSPACE."""
-    import tdub.rex
-    import tdub.config
-    if outdir == "auto":
-        outdir = PosixPath(workspace) / "matplotlib"
-    else:
-        outdir = PosixPath(outdir)
-    outdir.mkdir(exist_ok=True)
-    if impact:
-        tdub.rex.nuispar_impact_plot_top15(workspace)
-    if stacks:
-        tdub.config.init_meta_table()
-        tdub.config.init_meta_logy()
-        tdub.rex.plot_all_regions(workspace, outdir, stage="pre", show_chisq=chisq, n_test=n_test)
-        tdub.rex.plot_all_regions(workspace, outdir, stage="post", show_chisq=chisq, n_test=n_test)
-    return 0
-
-
-@cli.command("imp-tables", context_settings=dict(max_content_width=92))
-@click.argument("summary-file", type=click.Path(exists=True))
-def imp_tables(summary_file):
-    """Generate importance tables."""
-    import tdub.config
-    import json
-    from textwrap import dedent
-    tdub.config.init_meta_table()
-    summary = json.loads(PosixPath(summary_file).read_text())
-    imp_gain = summary["importances_gain"]
-    imp_split = summary["importances_split"]
-    names = imp_gain.keys()
-    imp_gain = [round(x, 4) for x in imp_gain.values()]
-    imp_split = [round(x, 4) for x in imp_split.values()]
-    imp_gain, names, imp_split = (list(reversed(t)) for t in zip(*sorted(zip(imp_gain, names, imp_split))))
-    print(dedent("""\
-    \\begin{table}[htbp]
-      \\begin{center}
-        \\caption{XXX}
-        \\label{XXX}
-        \\begin{tabular}{lcc}
-          \\toprule
-          Variable & Importance (gain) & Importance (split) \\\\
-          \\midrule"""))
-    for n, g, s in zip(names, imp_gain, imp_split):
-        print("      {} & {} & {} \\\\".format(tdub.config.PLOTTING_META_TABLE["titles"][n]["mpl"], g, s))
-    print(dedent("""\
-          \\bottomrule
-        \\end{tabular}
-      \\end{center}
-    \\end{table}
-    """))
-
-
 def run_cli():
     import tdub.config
     tdub.config.AVOID_IN_CLF_1j1b = []
