@@ -1,19 +1,21 @@
 """Art creation utilities."""
 
 # stdlib
-from typing import Any, Dict, Tuple, Optional, List, Union
+from typing import Dict, Tuple, Optional, List, Union
 import logging
 
 # external
 import matplotlib
 
-matplotlib.use("Agg")
+matplotlib.use("pdf")
 import matplotlib.pyplot as plt
 import numpy as np
+import pandas as pd
 
 # tdub
 from tdub import setup_logging
 import tdub.hist
+import tdub.root
 
 
 setup_logging()
@@ -110,8 +112,8 @@ def draw_atlas_label(
 
 
 def draw_uncertainty_bands(
-    uncertainty: Any,
-    total_mc: Any,
+    uncertainty: tdub.root.TGraphAsymmErrors,
+    total_mc: tdub.root.TH1,
     ax: plt.Axes,
     axr: plt.Axes,
     label: str = "Uncertainty",
@@ -122,10 +124,10 @@ def draw_uncertainty_bands(
 
     Parameters
     ----------
-    uncertainty : uproot_methods.classes.TGraphAsymmErrors.Methods
-        ROOT TGraphAsymmErrors from uproot with full systematic uncertainty.
-    total_mc : uproot_methods.classes.TH1.Methods
-        ROOT TH1 from uproot providing the full Monte Carlo prediction.
+    uncertainty : tdub.root.TGraphAsymmErrors
+        ROOT TGraphAsymmErrors with full systematic uncertainty.
+    total_mc : tdub.root.TH1
+        ROOT TH1 providing the full Monte Carlo prediction.
     ax : matplotlib.axes.Axes
         Main axis (where histogram stack is painted)
     axr : matplotlib.axes.Axes
@@ -136,9 +138,9 @@ def draw_uncertainty_bands(
         When total MC events are below threshold, zero contents and error.
 
     """
-    lo = np.hstack([uncertainty.yerrorslow, uncertainty.yerrorslow[-1]])
-    hi = np.hstack([uncertainty.yerrorshigh, uncertainty.yerrorshigh[-1]])
-    mc = np.hstack([total_mc.values, total_mc.values[-1]])
+    lo = np.hstack([uncertainty.ylo, uncertainty.ylo[-1]])
+    hi = np.hstack([uncertainty.yhi, uncertainty.yhi[-1]])
+    mc = np.hstack([total_mc.counts, total_mc.counts[-1]])
     ratio_y1 = 1 - (lo / mc)
     ratio_y2 = 1 + (hi / mc)
     set_to_zero = mc < zero_threshold
@@ -172,11 +174,11 @@ def draw_uncertainty_bands(
 
 
 def canvas_from_counts(
-    counts: Dict[str, Any],
-    errors: Dict[str, Any],
+    counts: Dict[str, np.ndarray],
+    errors: Dict[str, np.ndarray],
     bin_edges: np.ndarray,
-    uncertainty: Optional[Any] = None,
-    total_mc: Optional[Any] = None,
+    uncertainty: Optional[tdub.root.TGraphAsymmErrors] = None,
+    total_mc: Optional[tdub.root.TH1] = None,
     logy: bool = False,
     **subplots_kw,
 ) -> Tuple[plt.Figure, plt.Axes, plt.Axes]:
@@ -200,9 +202,9 @@ def canvas_from_counts(
         a dictionray pairing samples to bin count errors.
     bin_edges : array_like
         the histogram bin edges.
-    uncertainty : uproot_methods.base.classes.TGraphAsymmErrors.Methods, optional
+    uncertainty : tdub.root.TGraphAsymmErrors
         Uncertainty (TGraphAsym).
-    total_mc : uproot_methods.base.classes.TH1.Methods, optional
+    total_mc : tdub.root.TH1
         Total MC histogram (TH1D).
     subplots_kw : dict
         remaining keyword arguments passed to :py:func:`matplotlib.pyplot.subplots`.
@@ -283,7 +285,10 @@ def canvas_from_counts(
 
 
 def draw_impact_barh(
-    ax: plt.Axes, df: Any, hi_color: str = "skyblue", lo_color: str = "peru"
+    ax: plt.Axes,
+    df: pd.DataFrame,
+    hi_color: str = "steelblue",
+    lo_color: str = "mediumturquoise",
 ) -> Tuple[plt.Axes, plt.Axes]:
     """Draw the impact plot.
 
