@@ -11,7 +11,7 @@ from typing import Optional, Tuple, List, Union, Dict, Any
 # externals
 import matplotlib
 
-matplotlib.use("Agg")
+matplotlib.use("pdf")
 import joblib
 import numpy as np
 import matplotlib.pyplot as plt
@@ -30,6 +30,7 @@ from tdub.data import Region, features_for, quick_files, selection_for, selectio
 from tdub.frames import iterative_selection, drop_cols
 from tdub.hist import bin_centers
 from tdub.math import ks_twosample_binned
+import tdub.config
 
 setup_tdub_style()
 log = logging.getLogger(__name__)
@@ -146,8 +147,12 @@ class ResponseHistograms:
 
     def _eval_model(self, model, X_train, X_test):
         if self.response_type == "predict":
-            r_test = model.predict(X_test, raw_score=True)
-            r_train = model.predict(X_train, raw_score=True)
+            try:
+                r_test = model.predict(X_test, raw_score=True)
+                r_train = model.predict(X_train, raw_score=True)
+            except TypeError:
+                r_test = model.predict(X_test)
+                r_train = model.predict(X_train)
         elif self.response_type == "decision_function":
             r_test = model.decision_function(X_test)
             r_train = model.decision_function(X_train)
@@ -212,7 +217,9 @@ class ResponseHistograms:
         return round(float(self.ks_bkg[1]), 5)
 
     def draw(
-        self, ax: Optional[plt.Axes] = None, xlabel: Optional[str] = None,
+        self,
+        ax: Optional[plt.Axes] = None,
+        xlabel: Optional[str] = None,
     ) -> Tuple[plt.Figure, plt.Axes]:
         """Draw the response histograms.
 
@@ -232,7 +239,7 @@ class ResponseHistograms:
 
         """
         if ax is None:
-            fig, ax = plt.subplots(figsize=(4.5, 4))
+            fig, ax = plt.subplots(figsize=(5.06, 4.5))
         else:
             fig = ax.get_figure()
         bc = bin_centers(self.bins)
@@ -286,7 +293,10 @@ class ResponseHistograms:
         ax.legend(handles, labels, loc="upper right", ncol=1, frameon=False, numpoints=1)
         ax.set_ylabel("Arbitrary Units")
         draw_atlas_label(
-            ax, follow_shift=0.2, cme_and_lumi=False, extra_lines=["$tW$ BDT Training"]
+            ax,
+            cme_and_lumi=False,
+            extra_lines=["$tW$ BDT Training"],
+            follow_shift=0.22,
         )
         if xlabel is None:
             if self.response_type == "proba":
@@ -311,6 +321,7 @@ def prepare_from_root(
     use_tptrw: bool = False,
     use_trrw: bool = False,
     test_case_size: Optional[int] = None,
+    bkg_sample_frac: Optional[float] = None,
 ) -> Tuple[pd.DataFrame, np.ndarray, np.ndarray]:
     """Prepare the data to train in a region with signal and background ROOT files.
 
@@ -338,19 +349,21 @@ def prepare_from_root(
         See the parameter description for :py:func:`tdub.frames.iterative_selection`.
     use_tptrw : bool
         Apply the top pt reweighting factor.
-    use_tptrw : bool
+    use_trrw : bool
         Apply the top recursive reweighting factor.
     test_case_size : int, optional
         Prepare a small test case dataset using this many training and
         testing samples.
+    bkg_sample_frac : float, optional
+        Sample a fraction of the background data.
 
     Returns
     -------
-    pandas.DataFrame
+    :py:obj:`pandas.DataFrame`
        Event feature matrix.
-    numpy.ndarray
+    :py:obj:`numpy.ndarray`
        Event labels (0 for background; 1 for signal).
-    numpy.ndarray
+    :py:obj:`numpy.ndarray`
        Event weights.
 
     Examples
@@ -403,13 +416,14 @@ def prepare_from_root(
         use_campaign_weight=use_campaign_weight,
         use_tptrw=use_tptrw,
         use_trrw=use_trrw,
+        sample_frac=bkg_sample_frac,
     )
 
     if test_case_size is not None:
         if test_case_size > 5000:
             log.warn("why bother with test_case_size > 5000?")
-        sig_df = sig_df.sample(n=test_case_size, random_state=414)
-        bkg_df = bkg_df.sample(n=test_case_size, random_state=414)
+        sig_df = sig_df.sample(n=test_case_size, random_state=tdub.config.RANDOM_STATE)
+        bkg_df = bkg_df.sample(n=test_case_size, random_state=tdub.config.RANDOM_STATE)
 
     w_sig = sig_df.pop("weight_nominal").to_numpy()
     w_bkg = bkg_df.pop("weight_nominal").to_numpy()
@@ -445,6 +459,49 @@ def prepare_from_root(
     return df, y, w
 
 
+def persist_prepared_data(
+    out_dir: Union[str, os.PathLike],
+    df: pd.DataFrame,
+    labels: np.ndarray,
+    weights: np.ndarray,
+) -> None:
+    """Persist prepared data to disk.
+
+    The product of :py:func:`tdub.ml_train.prepare_from_root` is
+    easily persistable to disk; this function performs that task. If
+    the same prepared data is going to be used for multiple training
+    executations, one can save CPU cycles by saving the prepared data
+    instead of starting higher upstream with our ROOT ntuples.
+
+    Parameters
+    ----------
+    out_dir : str or os.PathLike
+        Directory to save output to.
+    df : pandas.DataFrame
+        Prepared DataFrame object.
+    labels : numpy.ndarray
+        Prepared labels.
+    weights : numpy.ndarray
+        Prepared weights.
+
+    Examples
+    --------
+    >>> from tdub.data import quick_files
+    >>> from tdub.train import prepare_from_root, persist_prepared_data
+    >>> qfiles = quick_files("/path/to/data")
+    >>> df, y, w = prepare_from_root(qfiles["tW_DR"], qfiles["ttbar"], "2j2b")
+    >>> persist_prepared_data("/path/to/output/data", df, y, w)
+
+    """
+    out_dir = PosixPath(out_dir)
+    out_dir.mkdir(exist_ok=True, parents=True)
+    df.to_hdf(out_dir / "df.h5", "df", mode="w", complevel=0)
+    np.save(out_dir / "labels.npy", labels)
+    np.save(out_dir / "weights.npy", weights)
+    selection_file = PosixPath(out_dir / "selection.txt")
+    selection_file.write_text(f"{df.selection_used}\n")
+
+
 def tdub_train_axes(
     learning_rate: float = 0.1,
     max_depth: int = 5,
@@ -487,7 +544,7 @@ def tdub_train_axes(
 
 def sklearn_gen_classifier(
     early_stopping_rounds: int = 10,
-    validation_fraction: float = 0.33,
+    validation_fraction: float = 0.20,
     train_axes: Optional[Dict[str, Any]] = None,
     **clf_params,
 ) -> BaseEstimator:
@@ -545,7 +602,11 @@ def sklearn_gen_classifier(
 
 
 def sklearn_train_classifier(
-    clf: BaseEstimator, X_train: Any, y_train: Any, w_train: Any, **fit_params,
+    clf: BaseEstimator,
+    X_train: Any,
+    y_train: Any,
+    w_train: Any,
+    **fit_params,
 ) -> BaseEstimator:
     """Train a Scikit-learn classifier.
 
@@ -617,12 +678,14 @@ def lgbm_train_classifier(
     X_train: Any,
     y_train: Any,
     w_train: Any,
-    validation_fraction: float = 0.33,
+    validation_fraction: float = 0.20,
     early_stopping_rounds: int = 10,
     **fit_params,
 ) -> BaseEstimator:
     """Train a LGBMClassifier.
 
+    Parameters
+    ----------
     clf : lightgbm.LGBMClassifier
         The classifier
     X_train : array_like
@@ -649,7 +712,7 @@ def lgbm_train_classifier(
         y_train,
         w_train,
         test_size=validation_fraction,
-        random_state=414,
+        random_state=tdub.config.RANDOM_STATE,
         shuffle=True,
     )
     return clf.fit(
@@ -663,6 +726,90 @@ def lgbm_train_classifier(
     )
 
 
+def xgb_gen_classifier(train_axes: Dict[str, Any] = None, **clf_params) -> BaseEstimator:
+    """Create a classifier using XGBoost.
+
+    Parameters
+    ----------
+    train_axes : dict[str, Any]
+        Values of required tdub training parameters.
+    clf_params : kwargs
+        Extra arguments passed to the constructor.
+
+    Returns
+    -------
+    xgboost.XGBClassifier
+        The classifier.
+
+    """
+    import xgboost as xgb
+
+    if train_axes is None:
+        train_axes = tdub_train_axes()
+    params = dict(
+        learning_rate=train_axes.get("learning_rate", 0.1),
+        max_depth=train_axes.get("max_depth", 5),
+        min_child_weight=train_axes.get("min_child_samples", 50),
+        reg_lambda=train_axes.get("reg_lambda", 0.0),
+    )
+    clf = xgb.XGBClassifier(booster="gbtree", n_estimators=500, **params)
+    for k, v in clf.get_params().items():
+        log.info(f"{k:>20} = {v}")
+    return clf
+
+
+def xgb_train_classifier(
+    clf: BaseEstimator,
+    X_train: Any,
+    y_train: Any,
+    w_train: Any,
+    validation_fraction: float = 0.20,
+    early_stopping_rounds: int = 10,
+    **fit_params,
+) -> BaseEstimator:
+    """Train a XGBClassifier.
+
+    clf : xgboost.XGBClassifier
+        The classifier
+    X_train : array_like
+        Training events matrix
+    y_train : array_like
+        Training event labels
+    w_train : array_like
+        Training event weights
+    validation_fraction : float
+        Fraction of training events to use in validation set.
+    early_stopping_rounds : int
+        Number of early stopping rounds to use in training.
+    fit_params : keyword arguments
+        Extra keyword arguments passed to the classifier.
+
+    Returns
+    -------
+    xgboost.XGBClassifier
+        The same classifier object passed to the function
+
+    """
+
+    X_t, X_v, y_t, y_v, w_t, w_v = train_test_split(
+        X_train,
+        y_train,
+        w_train,
+        test_size=validation_fraction,
+        random_state=tdub.config.RANDOM_STATE,
+        shuffle=True,
+    )
+    return clf.fit(
+        X_t,
+        y_t,
+        sample_weight=w_t,
+        eval_set=[(X_v, y_v)],
+        eval_metric="auc",
+        sample_weight_eval_set=[w_v],
+        early_stopping_rounds=early_stopping_rounds,
+    )
+
+
 def single_training(
     df: pd.DataFrame,
     labels: np.ndarray,
@@ -670,10 +817,11 @@ def single_training(
     train_axes: Dict[str, Any],
     output_dir: Union[str, os.PathLike],
     test_size: float = 0.40,
-    random_state: int = 414,
     early_stopping_rounds: Optional[int] = None,
     extra_summary_entries: Optional[Dict[str, Any]] = None,
     use_sklearn: bool = False,
+    use_xgboost: bool = False,
+    save_lgbm_txt: bool = False,
 ) -> None:
     """Execute a single training with some parameters.
 
@@ -694,15 +842,17 @@ def single_training(
         Directory to save results of training
     test_size : float
         Test size for splitting into training and testing sets
-    random_state : int
-        Random seed for
-        :py:func:`sklearn.model_selection.train_test_split`.
     early_stopping_rounds : int, optional
         Number of rounds to have no improvement for stopping training.
     extra_summary_entries : dict, optional
         Extra entries to save in the JSON output summary.
     use_sklearn : bool
         Use Scikit-learn's HistGradientBoostingClassifier.
+    use_xgboost : bool
+        Use XGBoost's XGBClassifier.
+    save_lgbm_txt : bool
+        Save fitted LGBM model to text file (ignored if either
+        ``use_sklearn`` or ``use_xgboost`` is ``True``).
 
     Returns
     -------
@@ -728,7 +878,10 @@ def single_training(
     ... )
 
     """
-    import lightgbm as lgbm
+    use_lgbm = not use_sklearn and not use_xgboost
+
+    if sum([use_sklearn, use_lgbm, use_xgboost]) != 1:
+        raise RuntimeError("BDT provider not defined properly.")
 
     starting_dir = os.getcwd()
     output_path = PosixPath(output_dir)
@@ -737,7 +890,12 @@ def single_training(
     os.chdir(output_path)
 
     X_train, X_test, y_train, y_test, w_train, w_test = train_test_split(
-        df, labels, weights, test_size=test_size, random_state=random_state, shuffle=True
+        df,
+        labels,
+        weights,
+        test_size=test_size,
+        random_state=tdub.config.RANDOM_STATE,
+        shuffle=True,
     )
 
     log.info(f"test size used: {test_size}")
@@ -756,8 +914,12 @@ def single_training(
             train_axes=train_axes,
         )
         sklearn_train_classifier(model, X_train, y_train, w_train)
-
-    else:
+    elif use_xgboost:
+        model = xgb_gen_classifier(train_axes=train_axes)
+        xgb_train_classifier(
+            model, X_train, y_train, w_train, early_stopping_rounds=early_stopping_rounds
+        )
+    elif use_lgbm:
         model = lgbm_gen_classifier(train_axes=train_axes)
         lgbm_train_classifier(
             model, X_train, y_train, w_train, early_stopping_rounds=early_stopping_rounds
@@ -765,6 +927,8 @@ def single_training(
 
     # Save the model
     joblib.dump(model, "model.joblib.gz", compress=("gzip", 3))
+    if use_lgbm and save_lgbm_txt:
+        model.booster_.save_model("model.txt")
 
     # ROC curve
     fig_roc, ax_roc = plt.subplots(figsize=(4.5, 4))
@@ -779,7 +943,8 @@ def single_training(
     importances_gain = {}
     importances_split = {}
     # Plot Importances
-    if not use_sklearn:
+    if use_lgbm:
+        import lightgbm as lgbm
         fig_imp, (ax_imp_gain, ax_imp_split) = plt.subplots(2, 1)
         lgbm.plot_importance(model, ax=ax_imp_gain, importance_type="gain", precision=2)
         lgbm.plot_importance(model, ax=ax_imp_split, importance_type="split", precision=2)
@@ -794,8 +959,8 @@ def single_training(
         feat_name = model.booster_.feature_name()
         imp_gain = np.array(imp_gain, dtype=np.float64)
         imp_split = np.array(imp_split, dtype=np.float64)
-        imp_gain *= (1.0 / np.sum(imp_gain))
-        imp_split *= (1.0 /  np.sum(imp_split))
+        imp_gain *= 1.0 / np.sum(imp_gain)
+        imp_split *= 1.0 / np.sum(imp_split)
         for n, g, s in zip(feat_name, imp_gain, imp_split):
             importances_gain[n] = float(g)
             importances_split[n] = float(s)
@@ -804,14 +969,19 @@ def single_training(
     proba_histograms = ResponseHistograms(
         "proba", model, X_train, X_test, y_train, y_test, w_train, w_test
     )
-    if not use_sklearn:
-        pred_histograms = ResponseHistograms(
-            "predict", model, X_train, X_test, y_train, y_test, w_train, w_test
-        )
-    else:
+    if use_sklearn:
         pred_histograms = ResponseHistograms(
             "decision_function", model, X_train, X_test, y_train, y_test, w_train, w_test
         )
+    elif use_xgboost:
+        pred_histograms = ResponseHistograms(
+            "predict", model, X_train, X_test, y_train, y_test, w_train, w_test
+        )
+    elif use_lgbm:
+        pred_histograms = ResponseHistograms(
+            "predict", model, X_train, X_test, y_train, y_test, w_train, w_test
+        )
+
     fig_pred, ax_pred = pred_histograms.draw()
     fig_proba, ax_proba = proba_histograms.draw()
     fig_pred.subplots_adjust(bottom=0.125, left=0.15)
@@ -842,10 +1012,14 @@ def single_training(
     summary["importances_split"] = importances_split
 
     if early_stopping_rounds is not None:
-        if use_sklearn:
+        if hasattr(model, "n_iter_"):
             summary["best_iteration"] = int(model.n_iter_)
-        else:
+        elif hasattr(model, "best_iteration"):
+            summary["best_iteration"] = int(model.best_iteration)
+        elif hasattr(model, "best_iteration_"):
             summary["best_iteration"] = int(model.best_iteration_)
+        else:
+            log.warn("best iteration undetected")
     if extra_summary_entries is not None:
         for k, v in extra_summary_entries.items():
             summary[k] = v
@@ -874,11 +1048,11 @@ def folded_training(
     information) are saved to ``output_dir``. The entries in the
     ``kfold_kw`` argument are forwarded to the
     :obj:`sklearn.model_selection.KFold` class for data preprocessing.
-    The default arguments that we use are:
+    The default arguments that we use are (random_state is controlled
+    by the tdub.config module):
 
     - ``n_splits``: 3
     - ``shuffle``: ``True``
-    - ``random_state``: 414
 
     Parameters
     ----------
@@ -927,7 +1101,7 @@ def folded_training(
     ...     {"verbose": 20},
     ...     "/path/to/train/output",
     ...     "2j2b",
-    ...     kfold_kw={"n_splits": 5, "shuffle": True, "random_state": 17}
+    ...     kfold_kw={"n_splits": 5, "shuffle": True},
     ... )
 
     """
@@ -959,7 +1133,7 @@ def folded_training(
     aucs = []
     importances = np.zeros((len(df.columns)))
     mean_fpr = np.linspace(0, 1, 100)
-    folder = KFold(**kfold_kw)
+    folder = KFold(**kfold_kw, random_state=tdub.config.RANDOM_STATE)
     fold_number = 0
     nfits = 0
     for train_idx, test_idx in folder.split(df):
@@ -1161,7 +1335,6 @@ def gp_minimize_auc(
     output_dir: Union[str, os.PathLike] = "_unnamed_optimization",
     n_calls: int = 15,
     esr: Optional[int] = 10,
-    random_state: int = 414,
 ):
     """Minimize AUC using Gaussian processes.
 
@@ -1212,7 +1385,12 @@ def gp_minimize_auc(
 
     df, labels, weights = prepare_from_root(tW_files, qfiles["ttbar"], region)
     X_train, X_test, y_train, y_test, w_train, w_test = train_test_split(
-        df, labels, weights, train_size=0.333, random_state=random_state, shuffle=True
+        df,
+        labels,
+        weights,
+        train_size=0.333,
+        random_state=tdub.config.RANDOM_STATE,
+        shuffle=True,
     )
     validation_data = [(X_test, y_test)]
     validation_w = w_test
@@ -1249,7 +1427,10 @@ def gp_minimize_auc(
 
     @use_named_args(dimensions=dimensions)
     def afit(
-        learning_rate, num_leaves, min_child_samples, max_depth,
+        learning_rate,
+        num_leaves,
+        min_child_samples,
+        max_depth,
     ):
         global _ifit
         global _best_fit
