@@ -57,7 +57,9 @@ def test_drop_jet2():
 
 def test_selection_augmented():
     files = [str(test_file_root / "testfile1.root"), str(test_file_root / "testfile3.root")]
-    df = iterative_selection(files, "(OS == True) & (reg1j1b == True) & (mass_lep1jet1 < 155)")
+    df = iterative_selection(
+        files, "(OS == True) & (reg1j1b == True) & (mass_lep1jet1 < 155)"
+    )
     sel_vars = set(selection_branches(df.selection_used))
     manual = {"OS", "reg1j1b", "mass_lep1jet1"}
     assert sel_vars == manual
